#!/usr/bin/perl
#
# This script will generate an grib inventory file (idx) for
# The input grib1 file. If all files if the set are in, 
# create a combined ctl file and idx file.
#
#
#
use Time::CTime;           # for strftime

sub usage;

$nargs = @ARGV;
if ($nargs < 2)  {
   usage ();
}

$grib_filename=shift;
$filelist_template=shift;
$make_combined_ctl_script=shift;

# Get date and cycle from filename

$grib_filename =~ /.*\/(.*)/;
$dirname=$1;
$grib_filename =~ /.*\/.*(\d{8})\/.*\.t(\d\d)z\..*/;
$yyyymmdd=$1;
$cycle=$2;
print "yyyymmdd=$yyyymmdd cycle=$cycle\n";


$dbn_root = $ENV{'DBNROOT'};
$data_root = $ENV{'DATAROOT'};

# Make the ctl, idx, and inv files

if (make_files( $grib_filename ) ) {

#  If we have all the files, make the combined ctl file.
if ( $template != "none") {

   if (check_for_file_list ($filelist_template, $yyyymmdd, $cycle)) {
   
      `$dbn_root/user/$make_combined_ctl_script $yyyymmdd$cycle $dirname`
   }
} else {
      `$dbn_root/user/$make_combined_ctl_script $yyyymmdd$cycle $dirname`
}

} else {
   print "make_files returned fals\n";
}


sub check_for_file_list() {

######################################################################
# Purpose:
#  Check that files are present on local host.  
#  If all are present, return 0.  If any are missing, return 1.
#  
#
# Input:
#       fileset table: file name containing a directory name and filenames
#
# Author: Paula Freeman, some cobbled from Richard Putt's script.
######################################################################
   
   $verbose=0;
   
   $fileTable= shift;
   $YYYYMMDD = shift;
   $CC = shift;
   
   
   # Define hash table
   
   %fileListHash=();
   
   
   # Open the table
   
   if (! open (TABLE,"$fileTable"))  {
      print "$0: Cannot open $fileTable: $!";
      exit 0;
   }
   
   # First line is the directory pattern
   
   $dirLine=<TABLE>;
   ($baseDir, $dirTime) = split(/\s+/, $dirLine);
   chomp $baseDir;
   $baseDir =~ s/YYYYMMDD/$YYYYMMDD/g;
   $baseDir =~ s/CC/$CC/g;
   
   $verbose && print "\nChecking for files in $baseDir \n";
   
   # The rest of the lines are the filenames.
   
   while ($filename=<TABLE>) {
      chomp $filename;
      $filename =~ s/CC/$CC/g;
      $fileListHash{$filename} = "1";
   }
   
   #foreach $key (keys %fileListHash) {
   #   print "$key\n";
   #}
   
   
   # Read the directory.
   # If directory doesn't exist, and it's still before the 
   # directory creation time limit, just exit.  If after
   # then generate and error.
   
   if (!opendir(DIR, $baseDir)) {
      print "$0: opendir($baseDir) failed: $!\n";
      exit 1;
   }
   @dirLines=readdir(DIR);
   closedir(DIR);
   
   foreach $f (@dirLines){
      chomp $f;
      $filePath = "$baseDir/$f";
      # next if !-f $filePath;
     if ( exists $fileListHash{$f} ){
       delete $fileListHash{$f} ;
     } else {
   #    print "WARNING: File not in table " . $hostname . ":" . $baseDir . "$filePath\n";
     }
   }
   
   
   # Print out names of missing files
   
   @missingFiles = sort keys %fileListHash;
   $missedFileCount = scalar( @missingFiles );
   
   #print "number of missing files: $missedFileCount\n";
   if ( $missedFileCount != 0) {
      print "\nFiles that are missing: \n\n";
      for $file ( sort keys( %fileListHash ) ){
        print "$baseDir/$file\n";
      }
      return 0;
   } else {
      $verbose && print "\nGOOD: No missing/late files.\n";
      return 1;
   }
   
}
   
sub usage {
      print "usage: $0: grib_filename file_list_template\n";
   cleanup;
   exit;
}

sub make_files { 

   $grib_filename = shift;
   
   $today=strftime("%Y%m%d", gmtime(time));
   $inv_log="$dbn_root/log/inv.log.$today";
   $ctl_log="$dbn_root/log/ctl.log.$today";
   $idx_log="$dbn_root/log/idx.log.$today";
   
   $wgrib="/usr/local/bin/wgrib";
   $grib2ctl="$dbn_root/user/grib2ctl.pl";
   $gribmap="/usr/local/bin/gribmap";
   
   $grib_inv_file = $grib_filename . ".inv";
#  $grib_ctl_file = $grib_filename . ".ctl";
#  $grib_idx_file = $grib_filename . ".idx";
   
   print "Generate inv file $grib_inv_file\n";
   `echo "Generate inv file $grib_inv_file"  >> $inv_log `;
   `$wgrib $grib_filename -s > $grib_inv_file 2>> $inv_log `;
   if ( -z $grib_inv_file) {
      `echo "Error making inv file for $grib_file"  >> $ctl_log`;
      unlink $grib_inv_file;
   }
   
#  print "Generate ctl file $grib_ctl_file\n";
#  `echo "Generate ctl file $grib_ctl_file"  >> $ctl_log`;
#  `$grib2ctl $grib_filename > $grib_ctl_file 2>> $ctl_log `;
#  if ( -z $grib_ctl_file) {
#     `echo "Error making ctl file $grib_ctl_file"  >> $ctl_log `;
#  } else {
#     print "Generate idx file $grib_idx_file\n";
#     `echo "Generate idx file $grib_idx_file"  >> $idx_log `;
#     `$gribmap -e -i $grib_ctl_file  >> $idx_log 2>&1 `;
#     if ( -z "${grib_file}.idx") {
#        `echo "Error making idx file for $grib_ctl_file"  >> $idx_log`;
#        return 0;
#     }
#  }
return 1 ;
}

